import { Getter } from '@loopback/core';
import { DataObject, Entity, Filter, Where } from '@loopback/repository';
import { Count } from '@loopback/repository/src/common-types';
import { Options } from 'loopback-datasource-juggler';
import { SequelizeCrudRepository, SequelizeDataSource } from 'loopback4-sequelize';
import { SoftDeleteEntity } from '../../models';
import { IUser } from '../../types';
export declare abstract class SequelizeSoftCrudRepository<E extends SoftDeleteEntity, ID, R extends object = {}> extends SequelizeCrudRepository<E, ID, R> {
    protected readonly getCurrentUser?: Getter<IUser | undefined> | undefined;
    constructor(entityClass: typeof Entity & {
        prototype: E;
    }, dataSource: SequelizeDataSource, getCurrentUser?: Getter<IUser | undefined> | undefined);
    find(filter?: Filter<E>, options?: Options): Promise<(E & R)[]>;
    findAll(filter?: Filter<E>, options?: Options): Promise<(E & R)[]>;
    findOne(filter?: Filter<E>, options?: Options): Promise<(E & R) | null>;
    findOneIncludeSoftDelete(filter?: Filter<E>, options?: Options): Promise<(E & R) | null>;
    findById(id: ID, filter?: Filter<E>, options?: Options): Promise<E & R>;
    findByIdIncludeSoftDelete(id: ID, filter?: Filter<E>, options?: Options): Promise<E & R>;
    updateAll(data: DataObject<E>, where?: Where<E>, options?: Options): Promise<Count>;
    count(where?: Where<E>, options?: Options): Promise<Count>;
    countAll(where?: Where<E>, options?: Options): Promise<Count>;
    delete(entity: E, options?: Options): Promise<void>;
    deleteAll(where?: Where<E>, options?: Options): Promise<Count>;
    deleteById(id: ID, options?: Options): Promise<void>;
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteHard(entity: E, options?: Options): Promise<void>;
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteAllHard(where?: Where<E>, options?: Options): Promise<Count>;
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteByIdHard(id: ID, options?: Options): Promise<void>;
    private getUserId;
}
